@@ -159,9 +159,9 @@ def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_
 
     Args:
         zodia_level:
-            zodiacal level, low, median, high, corresponding index from 1-3.
+            zodiacal level, 'low', 'med', 'high', corresponding index from 1-3.
         earthshine_level:
-            earth shine level, low, median, high, corresponding index from 1-3.
+            earth shine level, 'low', 'med', 'high', corresponding index from 1-3.
         wavearr:
             wave array to be returned.
 
@@ -227,16 +227,23 @@ class DetectorCountsRate(source.ModelCube):
 
 class DetectorCounts(object):
 
-    def __init__(self, config, signal):
+    def __init__(self, config, signal, obst=300, repn=20):
 
         instru = config['configuration']
 
         darkcube = np.zeros(shape=signal.source_counts.shape, dtype=float) + instru['dark']      # e/s/pix
-        readnoisecube = np.zeros(shape=signal.source_counts.shape, dtype=float) + instru['readout_noise']   # e/pix
-        nskycube = get_nsky(signal.ccdspec_wave)        # e/s/specsampling
 
-        repn = config['repn']
-        obst = config['obst']
+        readnoisecube = np.zeros(shape=signal.source_counts.shape, dtype=float) + instru['readout_noise']   # e/pix
+
+        # nskycube = get_nsky(signal.ccdspec_wave)        # e/s/specsampling
+        nskycube = get_background(config['zodiacal_level'],
+                                   config['earthshine_level'],
+                                   signal.ccdspec_wave,
+                                   instru['spaxel_xsize'],
+                                   instru['spaxel_ysize'])
+
+        # repn = config['repn']
+        # obst = config['obst']
         npixw = 2
         specsampling = 1
 
@@ -254,7 +261,7 @@ class calculation_snr(object):
     def __init__(self, config, straylight=False):
 
         signal_rate = DetectorCountsRate(config)
-        noise = DetectorCounts(config, signal_rate)
+        noise = DetectorCounts(config, signal_rate, obst=config['obst'], repn=config['repn'])
 
         self.source = noise.sourcenn
         self.darknoise = np.sqrt(noise.darkn)
@@ -313,56 +320,56 @@ class calculation_snr(object):
 
         report = self
 
-        fig, axs = plt.subplots(figsize=(12, 9))
+        fig, axs = plt.subplots(figsize=(7, 9))
         fig.subplots_adjust(hspace=0.35, wspace=0.45,
-                            left=0.05, right=0.97, top=0.97, bottom=0.05)
+                            left=0.10, right=0.97, top=0.97, bottom=0.05)
         cm = plt.cm.get_cmap('jet_r')
 
-        ax_recontructed_magmap = plt.subplot2grid((3, 4), (0, 0))
-        im = ax_recontructed_magmap.imshow(self.img2d, origin='low', cmap=cm)
-        ax_recontructed_magmap.set_xlabel('spaxel')
-        ax_recontructed_magmap.set_ylabel('spaxel')
-        ax_recontructed_magmap.set_title('reconstructed magmap')
-        cbar_ax = fig.add_axes([0.25, 0.75, 0.01, 0.2])
-        cbar = plt.colorbar(im, cax=cbar_ax, ticks=np.arange(20, 25, 2), orientation='vertical')
-        cbar.set_label(r'mag/arcsec$^{2}$', labelpad=-1)
-        cbar.ax.tick_params(labelsize=7)
+        # ax_recontructed_magmap = plt.subplot2grid((3, 4), (0, 0))
+        # im = ax_recontructed_magmap.imshow(self.img2d, origin='low', cmap=cm)
+        # ax_recontructed_magmap.set_xlabel('spaxel')
+        # ax_recontructed_magmap.set_ylabel('spaxel')
+        # ax_recontructed_magmap.set_title('reconstructed magmap')
+        # cbar_ax = fig.add_axes([0.25, 0.75, 0.01, 0.2])
+        # cbar = plt.colorbar(im, cax=cbar_ax, ticks=np.arange(20, 25, 2), orientation='vertical')
+        # cbar.set_label(r'mag/arcsec$^{2}$', labelpad=-1)
+        # cbar.ax.tick_params(labelsize=7)
 
-        ax_recontructed_snr = plt.subplot2grid((3, 4), (1, 0))
-        ind = np.argmin(abs(self.mockwave - 5000))
-        im = ax_recontructed_snr.imshow(self.snr[ind, :, :], origin='low', cmap=cm)
-        ax_recontructed_snr.set_xlabel('spaxel')
-        ax_recontructed_snr.set_ylabel('spaxel')
-        ax_recontructed_snr.set_title('reconstructed SNR@5000A')
-        cbar_ax = fig.add_axes([0.25, 0.41, 0.01, 0.2])
-        cbar = plt.colorbar(im, cax=cbar_ax, ticks=np.arange(0, 4, 1), orientation='vertical')
-        cbar.set_label('SNR', labelpad=-1)
-        cbar.ax.tick_params(labelsize=7)
+        # ax_recontructed_snr = plt.subplot2grid((3, 4), (1, 0))
+        # ind = np.argmin(abs(self.mockwave - 5000))
+        # im = ax_recontructed_snr.imshow(self.snr[ind, :, :], origin='low', cmap=cm)
+        # ax_recontructed_snr.set_xlabel('spaxel')
+        # ax_recontructed_snr.set_ylabel('spaxel')
+        # ax_recontructed_snr.set_title('reconstructed SNR@5000A')
+        # cbar_ax = fig.add_axes([0.25, 0.41, 0.01, 0.2])
+        # cbar = plt.colorbar(im, cax=cbar_ax, ticks=np.arange(0, 4, 1), orientation='vertical')
+        # cbar.set_label('SNR', labelpad=-1)
+        # cbar.ax.tick_params(labelsize=7)
 
-        points = [[15, 15], [15, 20], [15, 25]]
-        npoints = len(points)
+        # points = [[15, 15], [15, 20], [15, 25]]
+        # npoints = len(points)
 
         # for k in range(npoints):
 
         k = 0
-        i = points[k][0]
-        j = points[k][1]
-        ax_snr = plt.subplot2grid((3, 3), (k, 1), colspan=2)
-        ax_snr.plot(self.mockwave, self.snr[:, j, i], 'k-')
+        # i = points[k][0]
+        # j = points[k][1]
+        ax_snr = plt.subplot2grid((3, 3), (k, 0), colspan=3)
+        ax_snr.plot(self.mockwave, self.snr, 'k-')
         ax_snr.set_ylabel('SNR')
         ax_snr.set_xlabel('wavelength')
 
-        ax_noises = plt.subplot2grid((3, 3), (k + 1, 1), colspan=2)
-        ax_noises.plot(self.mockwave, self.sourcenoise[:, j, i], label='source_noise')
-        ax_noises.plot(self.mockwave, self.skynoise[:, j, i], label='sky_noise')
-        ax_noises.plot(self.mockwave, self.readnoise[:, j, i], label='readout_noise')
-        ax_noises.plot(self.mockwave, self.darknoise[:, j, i], label='dark_noise')
+        ax_noises = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
+        ax_noises.plot(self.mockwave, self.sourcenoise, label='source_noise')
+        ax_noises.plot(self.mockwave, self.skynoise, label='sky_noise')
+        ax_noises.plot(self.mockwave, self.readnoise, label='readout_noise')
+        ax_noises.plot(self.mockwave, self.darknoise, label='dark_noise')
         ax_noises.legend(loc=1)
         ax_noises.set_ylabel('different counts')
 
-        ax_recontructed_spec = plt.subplot2grid((3, 3), (k + 2, 1), colspan=2)
-        ax_recontructed_spec.plot(self.mockwave, self.mockflux[:, j, i], 'k-', label='mockflux')
-        ax_recontructed_spec.plot(self.mockwave, self.mockerror[:, j, i], 'b-', label='mockerr')
+        ax_recontructed_spec = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
+        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
+        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
         ax_recontructed_spec.set_ylabel('mock spec')
         ax_recontructed_spec.set_xlabel('wavelength')
 
@@ -442,7 +449,13 @@ class calculation_exptime(object):
         darkc = instru['dark']      # e/s/pix
         rn = instru['readout_noise']   # e/pix
         obst = config['obst']
-        nsky = get_nsky(ccdspec_wave)
+        # nsky = get_nsky(ccdspec_wave)
+        nsky = get_background(config['zodiacal_level'],
+                                  config['earthshine_level'],
+                                  ccdspec_wave,
+                                  instru['spaxel_xsize'],
+                                  instru['spaxel_ysize'])
+
         npixw = 2
         specsampling = 1
 
@@ -469,6 +482,96 @@ class calculation_exptime(object):
 
         self.exptime = tmp_obst * repn
 
+        #--------
+        noise = DetectorCounts(config, signal_rate, obst=config['obst'], repn=repn)
+
+        self.source = noise.sourcenn
+        self.darknoise = np.sqrt(noise.darkn)
+        self.readnoise = np.sqrt(noise.rnn2)
+        self.skynoise = np.sqrt(noise.skynn)
+        self.sourcenoise = np.sqrt(noise.sourcenn)
+        self.saturate_mask = (self.source > 65535)
+
+        if straylight == False:
+            self.straylight = np.zeros(shape=self.source.shape, dtype=float)
+        else:
+            # add 2% of the brightest source counts as straylight
+            surface_brightness = np.sum(self.source, 0)
+            ind_ycen, ind_xcen = np.where(surface_brightness == np.max(surface_brightness))
+            nstraylight = self.source[:, ind_ycen[0], ind_xcen[0]] * 0.02
+
+            self.straylight = nstraylight
+
+        self.totnoise = np.sqrt(self.darknoise ** 2 + self.readnoise ** 2 +
+                                self.skynoise ** 2 + self.sourcenoise ** 2 +
+                                self.straylight ** 2)
+        self.sysnoise = np.sqrt(self.darknoise ** 2 + self.readnoise ** 2 +
+                                self.skynoise ** 2 + self.straylight ** 2)
+        self.snr = noise.sourcenn / self.totnoise
+
+
+        # model spectrum
+        self.mockwave = signal_rate.wavearr
+        self.mockflux = np.random.normal(signal_rate.fluxcube, scale=signal_rate.fluxcube / self.snr)
+        self.mockerror = signal_rate.fluxcube / self.snr
+
+        # model image
+        self.img2d = signal_rate.mag2d
+
+
+    def save(self, file):
+
+        f = h5py.File(file, 'w')
+        f.create_dataset('source', data=self.source)
+        f.create_dataset('darknoise', data=self.darknoise)
+        f.create_dataset('readnoise', data=self.readnoise)
+        f.create_dataset('skynoise', data=self.skynoise)
+        f.create_dataset('sourcenoise', data=self.sourcenoise)
+        f.create_dataset('totnoise', data=self.totnoise)
+        f.create_dataset('sysnoise', data=self.sysnoise)
+        f.create_dataset('straylight', data=self.straylight)
+        f.create_dataset('snr', data=self.snr)
+        f.create_dataset('saturate_mask', data=self.saturate_mask)
+        f.create_dataset('mockwave', data=self.mockwave)
+        f.create_dataset('mockflux', data=self.mockflux)
+        f.create_dataset('mockerror', data=self.mockerror)
+        f.create_dataset('img2d', data=self.img2d)
+        f.close()
+
+
+    def plot(self, outfile):
+
+        report = self
+
+        fig, axs = plt.subplots(figsize=(7, 9))
+        fig.subplots_adjust(hspace=0.35, wspace=0.45,
+                            left=0.10, right=0.97, top=0.97, bottom=0.05)
+        cm = plt.cm.get_cmap('jet_r')
+
+        k = 0
+        # i = points[k][0]
+        # j = points[k][1]
+        ax_snr = plt.subplot2grid((3, 3), (k, 0), colspan=3)
+        ax_snr.plot(self.mockwave, self.snr, 'k-')
+        ax_snr.set_ylabel('SNR')
+        ax_snr.set_xlabel('wavelength')
+
+        ax_noises = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
+        ax_noises.plot(self.mockwave, self.sourcenoise, label='source_noise')
+        ax_noises.plot(self.mockwave, self.skynoise, label='sky_noise')
+        ax_noises.plot(self.mockwave, self.readnoise, label='readout_noise')
+        ax_noises.plot(self.mockwave, self.darknoise, label='dark_noise')
+        ax_noises.legend(loc=1)
+        ax_noises.set_ylabel('different counts')
+
+        ax_recontructed_spec = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
+        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
+        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
+        ax_recontructed_spec.set_ylabel('mock spec')
+        ax_recontructed_spec.set_xlabel('wavelength')
+
+        plt.savefig(outfile)
+        plt.close()
 
 
 def perform_calculation(config, calculation_mode='exptime2snr'):
