@@ -68,7 +68,7 @@ class read_template(object):
 
         elif template_filename.endswith('.txt'):
 
-            cat = pd.read_csv(template_filename, sep='\s+', header=None)
+            cat = pd.read_csv(template_filename, sep='\s+', header=None)    # separate by blank space
             template_wave = cat[0]      # unit should be in Angstrom
             template_flux = cat[1]      # unit should be in erg/s/cm^2/A
 
